/* Made by Monkey Machine */

package com.mkeym.customsec.ejb;

import java.lang.reflect.Method;
//import java.util.Random;
import java.security.Principal;
import java.util.Iterator;
import java.util.Enumeration;
import java.security.acl.Group;
import javax.ejb.EJBContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import org.apache.log4j.Category;

/**
* A typed security proxy class for the TradingAccountBean.
* Here we use the alternative approach of implementing methods with the same signtature as the bean home and remote interfaces.
*/
public class TypedSecurityProxy  {
	private Category _log = Category.getInstance(getClass().getName());

/* There is only one proxy created per container (i.e. not per bean), so we use a ThreadLocal store the invocation context. */

	private ThreadLocal _ctx = new ThreadLocal();

	/* we use another ThreadLocal to cache the bean instance */
	private ThreadLocal _bean = new ThreadLocal();


	public void init(Class beanHome, Class beanRemote, Object securityMgr) throws InstantiationException {
		_log.info("Security Proxy Initialising for " + beanRemote.getName());
	}

	/** Called prior to any method invocation to set the current EJB context. */
	public void setEJBContext(EJBContext ctx) {
		_log.info("setEJBContext " + ctx);
		_ctx.set(ctx);
	}

	/* If AbstractSecurityProxy is being used as the base class of the proxy wrapper, setBean is called to notify us which bean is being invoked */
	public void setBean(Object bean) {
		_log.info("setBean " + bean);
		if (!(bean instanceof TradingAccountBean))
			throw new SecurityException("Invalid bean instance for security proxy");
		_bean.set(bean);
	}

	/**
	 * Called to allow the security proxy to perform any custom security checks required for the EJB remote interface method.
	 * @param m, the EJB remote interface method that is being invoked
	 * @param bean, the EJB implementation class instance
	 */
	public void invoke(Method m, Object[] args, Object bean) throws SecurityException {
		EJBContext ctx = (EJBContext)_ctx.get();
		String caller = ctx.getCallerPrincipal().getName();
		TradingAccountBean account = (TradingAccountBean)bean;
		String operation = m.getName();
		_log.info("invoke " + operation + " called by " + caller);

		// The implementation of the defined policies.
		if (operation.equals("buy") || operation.equals("sell")) {
			int qty = ((Integer)args[1]).intValue();
			// 1. A trader can only buy or sell shares through their own account.
			if (!account.getAccountId().equals(caller)) {
				_log.warn("method " + operation + " called on account " + account.getAccountId() +
				    " by invalid user " + caller);
				throw new SecurityException("caller is not the account owner");
			}
			int value = qty * 100; // in our kid-on example all the shares cost 100.
			// 2. A single transaction must have a value less than or equal to 50 million.
			if (value > 5e7) {
				_log.warn(operation + " invoked with too high transaction value: " + value);
				throw new SecurityException("transaction value is too large");
			}
			int newBalance = account.getBalance() + (operation.equals("sell") ? value : -value);
			// 3. The balance of an account cannot exceed 100 million or go below -100 million.
			if (Math.abs(newBalance) >= 1e8) {
				_log.warn(operation + " would exceed bounds on account balance: " + newBalance);
				throw new SecurityException("balance exceeded");
			}
		}
	}
	/** called when home.create is invoked */
	public String create(String accountId) {
		EJBContext ctx = (EJBContext)_ctx.get();
		String caller = ctx.getCallerPrincipal().getName();
		_log.info("'create' called by " + caller + " with account Id = "+ accountId );
		// The return value is irrelevant here
		return null;
	}
	/** bean method */
	public void sell(String stock, int qty) {
		TradingAccountBean bean = (TradingAccountBean)_bean.get();
		EJBContext ctx = (EJBContext)_ctx.get();
		String caller = ctx.getCallerPrincipal().getName();
		_log.info("'sell' called by " + caller + " on account "+ bean.accountId );
	}
	/** bean method */
	public void buy(String stock, int qty) {
		TradingAccountBean bean = (TradingAccountBean)_bean.get();
		EJBContext ctx = (EJBContext)_ctx.get();
		String caller = ctx.getCallerPrincipal().getName();
		_log.info("'buy' called by " + caller + " on account "+ bean.accountId );
	}
}
